@@ -1,4 +1,4 @@
-//! A smart pointer with many reference-counted weak references.
+//! A `Box` with weak references.
 //!
 //! A [`RefBox`] is a smart pointer that owns the data, just like a standard
 //! [`Box`]. Similarly, a RefBox cannot be cloned cheaply, and when it is
@@ -17,10 +17,9 @@
 //! zero, one or many weak pointers and all borrows are mutable. This means
 //! there can only be one borrow active at any given time. But in return,
 //! RefBox uses less memory, is faster to borrow from, and a Ref does not need
-//! to be upgraded to a RefBox in order to access the data. In fact, upgrading
-//! is not possible at all.
+//! to be upgraded to a RefBox in order to access the data.
 //!
-//! Note: this crate is currently **experimental** and requires Nightly Rust.
+//! Note: this crate is currently **experimental**.
 //!
 //! [`Rc`]: std::rc::Rc
 //! [`Weak`]: std::rc::Weak
@@ -58,11 +57,19 @@
 //!     assert_eq!(*borrow, 100);
 //! }
 //! ```
+//!
+//! # Optional Features
+//!
+//! * **cyclic**: Enables the `RefBox::new_cyclic()` method. This allows you to
+//!   create data structures that contain weak references to (parts of)
+//!   themselves in one go. Requires the Nightly feature [layout_for_ptr].
+//!
+//! [layout_for_ptr]: https://github.com/rust-lang/rust/issues/69835
 
 // The optional "cyclic" feature, which activates the `RefBox::<T>::new_cyclic()`
 // method, requires the Nightly feature "layout_for_ptr", as we need to be able
 // to get the layout of `T` through a raw pointer in order to deallocate it.
-#![feature(layout_for_ptr)]
+#![cfg_attr(feature = "cyclic", feature(layout_for_ptr))]
 
 mod internals;
 
@@ -178,6 +185,8 @@ impl<T> RefBox<T> {
     ///
     /// Note: if you try to borrow the data in the closure, you will get a
     /// [`BorrowError::Dropped`] error.
+    /// 
+    /// Note: only available if the `cyclic` optional feature is enabled.
     ///
     /// # Examples
     ///
@@ -201,6 +210,7 @@ impl<T> RefBox<T> {
     ///     }
     /// });
     /// ```
+    #[cfg(feature = "cyclic")]
     pub fn new_cyclic<F: FnOnce(&Ref<T>) -> T>(op: F) -> Self {
         internals::new_cyclic_refbox(op)
     }
@@ -308,6 +318,16 @@ impl<T: ?Sized> RefBox<T> {
         self.ptr.as_ref().data_mut()
     }
 
+    /// Returns a raw pointer to `T`.
+    pub fn as_ptr(&self) -> *const T {
+        let ptr = self.ptr.as_ptr();
+
+        // SAFETY (1): ptr is safe to dereference, see Self::heap().
+        // SAFETY (2): UnsafeCell is `#[repr(transparent)]`, which means
+        // a pointer to the cell is also a pointer to its only field.
+        unsafe { std::ptr::addr_of!((*ptr).data) as *const T }
+    }
+
     /// Turns the `RefBox` into a raw pointer.
     pub fn into_raw(self) -> *mut RefBoxHeap<T> {
         let ptr = self.ptr.as_ptr();
@@ -488,7 +508,7 @@ impl<T: ?Sized> Ref<T> {
     ///
     /// 1. Ensure there are no mutable references to `T`.
     /// 2. Ensure `T` is fully initialized (don't use this in
-    ///    [`RefBox::new_cyclic`]).
+    ///    `RefBox::new_cyclic`).
     /// 3. Ensure the owning `RefBox` is alive for the entire lifetime
     ///    of the returned reference.
     pub unsafe fn get_unchecked(&self) -> &T {
@@ -502,13 +522,24 @@ impl<T: ?Sized> Ref<T> {
     ///
     /// 1. Ensure there are no other references to `T`.
     /// 2. Ensure `T` is fully initialized (don't use this in
-    ///    [`RefBox::new_cyclic`]).
+    ///    `RefBox::new_cyclic`).
     /// 3. Ensure the owning `RefBox` is alive for the entire lifetime
     ///    of the returned reference.
     pub unsafe fn get_mut_unchecked(&mut self) -> &mut T {
         self.ptr.as_ref().data_mut()
     }
 
+    /// Returns a raw pointer to `T`.
+    pub fn as_ptr(&self) -> *const T {
+        let ptr = self.ptr.as_ptr();
+
+        // SAFETY (1): ptr is safe to dereference, see Self::heap().
+        // SAFETY (2): UnsafeCell is `#[repr(transparent)]`, which means
+        // a pointer to the cell is also a pointer to its only field.
+        // SAFETY (3): data could be uninitialized, so use addr_of.
+        unsafe { std::ptr::addr_of!((*ptr).data) as *const T }
+    }
+
     /// Turns the `Ref` into a raw pointer.
     pub fn into_raw(self) -> *mut RefBoxHeap<T> {
         let ptr = self.ptr.as_ptr();
@@ -642,6 +673,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "cyclic")]
     fn rc_new_cyclic() {
         let rc = RefBox::new_cyclic(|_weak| 13579);
         assert_eq!(unsafe { *rc.get_unchecked() }, 13579);
@@ -651,6 +683,7 @@ mod tests {
     /// The refcount in the closure should be 1, and the refcount after
     /// creation should be 0.
     #[test]
+    #[cfg(feature = "cyclic")]
     fn rc_new_cyclic_refcount() {
         let rc = RefBox::new_cyclic(|weak| {
             assert_eq!(weak.refcount(), 1);
@@ -662,6 +695,7 @@ mod tests {
     /// The Ref in the closure should point to the same instance as the
     /// returned RefBox.
     #[test]
+    #[cfg(feature = "cyclic")]
     fn rc_new_cyclic_ptr_eq() {
         let mut out_weak = None;
         let rc = RefBox::new_cyclic(|weak| out_weak = Some(weak.clone()));
@@ -672,6 +706,7 @@ mod tests {
     /// MIRI: A panic in the closure of `new_cyclic` should not leak memory.
     #[test]
     #[should_panic]
+    #[cfg(feature = "cyclic")]
     fn rc_new_cyclic_does_not_leak() {
         let rc1 = RefBox::new_cyclic(|_weak| {
             panic!("panic in closure!");
@@ -956,6 +991,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "cyclic")]
     fn borrowing_in_cyclic_fails() {
         let rc = RefBox::new_cyclic(|weak| {
             let borrow = weak.try_borrow_mut_or_else(|| "was borrowed", || "was dropped");
@@ -965,6 +1001,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "cyclic")]
     fn cloning_weak_in_cyclic_increases_refcount() {
         let rc = RefBox::new_cyclic(|weak| {
             let weak2 = weak.clone();
@@ -1007,4 +1044,32 @@ mod tests {
         drop(rc1);
         drop(rc2);
     }
+
+    /// Test if `RefBox::as_ptr` returns the right pointer. Run with MIRI.
+    #[test]
+    fn refbox_as_ptr() {
+        let rc1 = RefBox::new(123456);
+        let ptr = rc1.as_ptr();
+
+        let heap = unsafe { rc1.ptr.as_ref() };
+        let real_ptr = heap.data.get();
+
+        assert_eq!(ptr, real_ptr);
+        drop(rc1);
+    }
+
+    /// Test if `Ref::as_ptr` returns the right pointer. Run with MIRI.
+    #[test]
+    fn ref_as_ptr() {
+        let rc1 = RefBox::new(123456);
+        let weak = rc1.create_ref();
+        let ptr = weak.as_ptr();
+
+        let heap = unsafe { weak.ptr.as_ref() };
+        let real_ptr = heap.data.get();
+
+        assert_eq!(ptr, real_ptr);
+        drop(weak);
+        drop(rc1);
+    }
 }
